#!/usr/bin/perl -w

# LSB Test Execution Framework
# Controlled Test Run Module (run_tests.pl)
#
# Copyright (C) 2007-2008 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

use Fcntl ":flock";

my $tests_dir = shift;
my $res_dir = shift;
my $profile = shift;
if (!$tests_dir or !$res_dir or !$profile) {
	die "Usage: $0 <TESTS_DIR> <RESULTS_DIR> <PROFILE>\n";
}

unlink("/tmp/tests_status.8888");
unlink("/tmp/tests_output.8888");
unlink("/tmp/tests_output.8888.old");

my $test_child = fork();
my $res;
if (!defined($test_child)) {
	die "Cannot create child process!";
}
elsif ($test_child == 0) {
	my $cert = "";
	my $first_line = "PID: ".getppid().", CHLD_PID: $$";
	if (open(PROFILE, $profile)) {
		while (<PROFILE>) {
			if (m/^CERTIFY:\s*(\d+)$/) {
				$first_line .= " (cert)";
				last;
			}
		}
		close(PROFILE);
	}
	system("echo \"$first_line\" >/tmp/tests_output.8888");
	exec("$tests_dir/lsb-tef.pl --webui -f \"$profile\" --results-dir=\"$res_dir\" >>/tmp/tests_output.8888 2>&1");
}

waitpid($test_child, 0);
$res = $?;
open(FILE_STAT, ">/tmp/tests_status.8888") or die "Could not open status file ‘/tmp/tests_status.8888’ for writing:\n$!";
flock(FILE_STAT, LOCK_EX);
print FILE_STAT "$res\n";
if ($res == 0) {
	my $tmp = `tail -n 1 $res_dir/HISTORY`;
	$tmp =~ s/[\r\n]//g;
	my $fail = "";
	if ($tmp) {
		if (-d "$res_dir/$tmp") {
			my $dir_name = "$res_dir/$tmp/test_logs";
			if (!-d $dir_name and !mkdir($dir_name)) {
				$fail = "Could not create directory ‘$dir_name’:\n$!";
			}
			elsif (rename("/tmp/tests_output.8888", "$dir_name/tests_output.8888")) {
				print FILE_STAT "$tmp\n";
			}
			else {
				$fail = "Could not move output file to ‘$dir_name’!\n";
			}
		}
		else {
			$fail = "Non-existent directory ‘$tmp’ specified in the HISTORY file!\n";
		}
	}
	else {
		$fail = "No directory specified in the HISTORY file!\n";
	}
	if ($fail) {
		rename("/tmp/tests_output.8888", "/tmp/tests_output.8888.old");
		print STDERR $fail;
	}
}
flock(FILE_STAT, LOCK_UN);
close(FILE_STAT);
